/*
	Projector control routines
	2/22/2019 Scott Grodevant for Optel, Inc.

	control of AK-02-00277_B using the DLPC3470 via its 100 KHz I2C Bus.


	Signals:

	GPIO#	Signal		Use
	19		PROJ_ON		Enables projector
	20		PROJ_IRQ	Projector interrupt (active high)
	21		TRIG2		From DLPC3470 GPIO7
	45		PATRDY		DLPC pattern ready (pattern data is loaded from Flash)
	25		FPIO4		DLPC 3DR (rising edge triggers next pattern display)
 */

#include <stdint.h>
#include "cyu3system.h"
#include <cyu3error.h>
#include <cyu3i2c.h>
#include <cyu3spi.h>
#include <cyu3utils.h>
#include <cyu3gpio.h>
#include <cyu3imagesensor.h>
#include "cyu3uart.h"
#include "cyu3pib.h"
#include "cyu3mipicsi.h"
#include <cyu3vic.h>


#include "sysconfig.h"
#include "projector.h"


bool		opened = false;														//	project is "open"
bar_types	testpat = bar_none;													//	current test pattern
uint32_t	oldi;																//	old interrupt state


//	Note that the DLPC 3470's I2C port is 100 KHz, our camera runs at 400 KHz, so we must toggle the
//	clock rate for the DLP, and disable interrupts to prevent communication with the camera sensor.

//	Here are the lowest-level I/O routines.  These must be called with I2C bus @ 100 KHz and
//	interrupts off.

//	Read count bytes from the projector reg into buf

CyU3PReturnStatus_t projread( uint8_t reg, uint8_t count, uint8_t *buf )
{
    CyU3PReturnStatus_t		status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t		preamble;

	preamble.buffer[0] = AP;													//	Slave address: write operation
	preamble.buffer[1] = reg;													//	reg to read
	preamble.buffer[3] = AP | 1;												//	Slave address: Read operation
	preamble.length = 3;
	preamble.ctrlMask = 0b0000000000000100;										//	repeated start after read command

	status = CyU3PI2cReceiveBytes( &preamble, buf, count, 0 );					//	read from reg
	CyU3PThreadSleep(1);

	if ( status != CY_U3P_SUCCESS ) return( false );

#ifdef PROJ_DEBUG
	CyU3PDebugPrint(4, "Projector read failed addr=0x%x (%X)\r\n", reg, status);
#endif
    return( status == CY_U3P_SUCCESS );
}

//	Write count bytes to projector register reg from buf.

CyU3PReturnStatus_t projwrite( uint8_t reg, uint16_t count, uint8_t *buf )
{
    CyU3PReturnStatus_t		status = CY_U3P_SUCCESS;
    CyU3PI2cPreamble_t		preamble;

	preamble.buffer[0] = AP;													//	Slave address: write operation
	preamble.buffer[1] = reg;													//	projector register
	preamble.length = 2;
	preamble.ctrlMask = 0b0000000000000000;

	status = CyU3PI2cTransmitBytes( &preamble, buf, count, 3 );
	CyU3PThreadSleep(1);
	if (status == CY_U3P_SUCCESS) return( false );

#ifdef PROJ_DEBUG
	CyU3PDebugPrint(4,"Projector Write Failed @ 0x%x (0x%x)\r\n", reg, status );
#endif
    return( status == CY_U3P_SUCCESS );
}


bool openprojector( void )
{
	CyU3PReturnStatus_t		status = CY_U3P_SUCCESS;

	if ( opened )
	{
		CyU3PDebugPrint(4,"Sequence error: projector has already been opened\r\n" );
		return( false );
	}

	opened = true;
//    status = CyU3PMipicsiInitializeI2c( CY_U3P_MIPICSI_I2C_100KHZ );			//	100 KHz
    if( CyU3PMipicsiInitializeI2c( CY_U3P_MIPICSI_I2C_100KHZ ) != CY_U3P_SUCCESS )
    {
		CyU3PDebugPrint(4,"Can't set I2C for 100 KHz\r\n" );
    	return( false );
    }

	oldi = CyU3PVicDisableAllInterrupts();
	return( status == CY_U3P_SUCCESS );
}

//	restore the i2c bus to 400 KHz and restore interrupts.

bool closeprojector( void )
{
	CyU3PReturnStatus_t		status;

	if ( !opened )
	{
		CyU3PDebugPrint(4,"Sequence error: projector is not opened\r\n" );
		return( false );
	}
	opened = false;
	CyU3PVicEnableInt( oldi );													//	restore original int state
    status = CyU3PMipicsiInitializeI2c( CY_U3P_MIPICSI_I2C_400KHZ );			//	back to 400 KHz
    return( status == CY_U3P_SUCCESS );
}


bool	projectorinit( void )
{
	uint8_t					buf[ 5 ];
	CyU3PReturnStatus_t		status;

	if ( !openprojector() ) return( false );									//	setup hardware to access the projector

    if ( ( status = projread( 0xD4, 1, buf ) ) != CY_U3P_SUCCESS )
    {
#ifdef PROJ_DEBUG
    	CyU3PDebugPrint(4,"Projector read failed @ 0x%x (0x%x)\r\n", 0xD4, status );
    	return( false );
#endif
    }
    else
    	CyU3PDebugPrint(4,"Projector ID is 0x%x (0x%x)\r\n", buf );


    //	do other stuff here...


    return( closeprojector() );
}


//	load splash screen image

bool	loadscreen( int screen, int addr, int bytes, byte *data )
{
	if ( !openprojector() ) return( false );									//	setup hardware to access the projector
    return( closeprojector() );
}

//	retrieve a splash screen image

bool	getscreen( int screen, int addr, int bytes, byte *data )
{
	if ( !openprojector() ) return( false );									//	setup hardware to access the projector

    return( closeprojector() );
}

//	prepare to show a splash screen image

bool	setscreen( int screen )
{
	if ( !openprojector() ) return( false );									//	setup hardware to access the projector

    return( closeprojector() );
}

//	show the next splash screen

bool	nextscreen( void )
{
	if ( !openprojector() ) return( false );									//	setup hardware to access the projector

    return( closeprojector() );
}


//	Splash screen access isn't supported until REV E hardware.  For now we can
//	use the 3470 horizontal and vertical bar patterns.

bool setbars( bar_types type, int on, int off )
{
	byte	data[ 6 ] = { type, 0b111000, 0, 0, 0, 0 };							//	color is FG: white, BG: black
	int		siz = 0;
	bool	status = true;

	testpat = type;

//	if ( !openprojector() ) return( false );									//	setup hardware to access the projector

	switch ( type )
	{
	case bar_none:
		data[ 0 ] = 0xFF;
//		projwrite( 5, 1, data );
		siz = 0;																//	nothing to write to projector reg 0xB
		break;

	case bar_solid:
		siz = 2;
		break;

	case bar_hramp:
	case bar_vramp:
		siz = 4;
		data[ 2 ] = 0;
		data[ 3 ] = 255;
		break;

	case bar_horiz:
	case bar_diag:
	case bar_vert:
		siz = 4;
		data[ 2 ] = on;															//	FG line width
		data[ 3 ] = off;														//	BG line width
		break;


	case bar_grid:
		siz = 6;
		data[ 2 ] = on;
		data[ 3 ] = off;
		data[ 4 ] = on;
		data[ 5 ] = off;
		break;

	case bar_checkerboard:
		siz = 6;
		data[ 2 ] = ( on & 0xFF );												//	# of horizontal checkers, LSB
		data[ 3 ] = ( ( on >> 8 ) & 0xFF );										//	horizontal checkers, MSB
		data[ 4 ] = ( off & 0xFF );												//	vertical checkers, LSB
		data[ 5 ] = ( ( off >> 8 ) & 0xFF );									//	vertical checkers, MSB
		break;

	case bar_vertcolor:
		siz = 2;
		break;

	default:
    	CyU3PDebugPrint(4,"Projector setbars: unknown pattern type @ %d\r\n", type );
		status = CY_U3P_SUCCESS + 1;
	}

//	if ( status && siz > 0 ) status = projwrite( 0xB, siz, data );				//	set projector test pattern
//	closeprojector();															//	close communication
    return( status );
}


bool getbars( bar_types *bars, int *on, int *off )
{
	byte	buf[ 6 ] = { 0, 0, 0, 0, 0, 0 };
	bool	status;

	if ( !openprojector() ) return( false );									//	setup hardware to access the projector

	if ( ( status = projread( 0xC, sizeof( buf ), buf ) ) == CY_U3P_SUCCESS )
	{
		*bars = buf[ 0 ];
		*on = buf[ 2 ] | ( buf[ 3 ] << 8 );
		*off = buf[ 4 ] | ( buf[ 5 ] << 8 );
	}
	else
		*bars = bar_solid;

    return( status && closeprojector() );
}

